#######################################################################
# Copyright (c) 2023 Juan Hern�ndez, FlowJo, LLC
#
# License
# The software is distributed under the terms of the 
# Artistic License 2.0
# http://www.r-project.org/Licenses/Artistic-2.0
# 
# Disclaimer
# This software and documentation come with no warranties of any kind.
# This software is provided "as is" and any express or implied 
# warranties, including, but not limited to, the implied warranties of
# merchantability and fitness for a particular purpose are disclaimed.
# In no event shall the  copyright holder be liable for any direct, 
# indirect, incidental, special, exemplary, or consequential damages
# (including but not limited to, procurement of substitute goods or 
# services; loss of use, data or profits; or business interruption)
# however caused and on any theory of liability, whether in contract,
# strict liability, or tort arising in any way out of the use of this 
# software.
######################################################################

## For debugging purposes
sessionInfo()

## For debugging to see what FlowJo is passing to the script.
##
## cat("FJ-SCRIPT-DIR			     = FJ_SCRIPT_DIR \n")
## cat("FJ-OUTPUT-DIR			     = FJ_OUTPUT_DIR \n")
## cat("FJ-BEST                		 = FJ_BEST \n")
## cat("FJ-CELLS                		 = FJ_CELLS \n")
## cat("FJ-DETECTORS                  = FJ_DETECTORS \n")
## cat("FJ-FLUOROCHROMES              = FJ_FLUOROCHROMES \n")
## cat("FJ-REPORT              		 = FJ_REPORT \n")

# Install function for packages
loadPackages <-function(...) {
  libs<-unlist(list(...))
  req<-unlist(lapply(libs,require,character.only=TRUE))
  need<-libs[req==FALSE]
  if(length(need)>0){
    install.packages(need, repos="https://cran.r-project.org")
    lapply(need,require,character.only=TRUE)
  }
}

loadBioc<-function(...) {
  libs<-unlist(list(...))
  req<-unlist(lapply(libs,require,character.only=TRUE))
  need<-libs[req==FALSE]
  if(length(need)>0){
    BiocManager::install(need)
    lapply(need,require,character.only=TRUE)
  }
}

loadPackages(c('BiocManager', 'patchwork', 'readxl', 'ggplot2', 'dplyr', 'stats', 'rlang', 'stringr','utils','methods','data.table'))

loadBioc(c('Biobase','PeacoQC','flowDensity','flowCore'))

setwd("FJ_SCRIPT_DIR")
pdf(file="FJ_REPORT")

source(file = "R/estimate_brightness.R")
source(file = "R/estimate_gate.R")
source(file = "R/estimate_SI.R")
source(file = "R/filter_plot.R")
source(file = "R/find_cutoff.R")
source(file = "R/find_plateau.R")
source(file = "R/plot_brightness.R")
source(file = "R/plot_voltration.R")
source(file = "R/prep_FCS.R")
source(file = "R/sample_cells.R")

getwd()

#variables
bg_data <- FJ_DATA
best <- FJ_BEST 
detectors <- FJ_DETECTORS
fls <- FJ_FLUOROCHROMES
cells_file <- FJ_CELLS

#script
SI <- data.frame(
  matrix(
    nrow = 0,
    ncol = 16,
    dimnames = list(
      NULL,
      c(
        "ID", "Fluorochrome","Detector","File", "Group","Cutoff", "MFI_pos", "MFI_neg",
        "Min_neg", "Max_neg", "rSD",
        "SI", "Voltage", "Pos_count","Neg_count", "Pctg_OutOfRange"
      )
    )
  ),
  check.names = FALSE
)

cells <- data.frame(matrix(NA,
                           nrow = 0,
                           ncol = 2,
                           dimnames = list(NULL, c("ID", "Value"))
))

if(is.na(cells_file) != TRUE) {
   	cells <-rbind(cells, read.table(cells_file,header=FALSE,sep='\t', col.names = c('ID', 'Value')))
}

for(group in bg_data){
  for(r in group){
    meta = data.frame(matrix(NA, nrow = 1, ncol = 0))
    meta["ID"] <- r[1]
    meta["Fluorochrome"] <- r[2]
    meta["Detector"] <- r[3]
    meta["File"] <- r[4]
    meta["Group"] <- r[5]
    meta["Cutoff"] <- as.numeric(r[9])
    meta["MFI_pos"] <- as.numeric(r[6])
    meta["MFI_neg"] <- as.numeric(r[7])
    meta["Min_neg"] <- as.numeric(r[8])
    meta["Max_neg"] <- as.numeric(r[9])
    meta["rSD"] <- as.numeric(r[10])
    meta["SI"] <- as.numeric(r[11])
    meta["Voltage"] <- as.numeric(r[12])
    meta["Pos_count"] <- as.numeric(r[13])
    meta["Neg_count"] <- as.numeric(r[14])
    meta["Pctg_OutOfRange"] <- as.numeric(r[15])
    rownames(meta) <- r[1]
    SI <- rbind(SI, meta)
    
    if(is.na(r[16]) != TRUE) {
    	cells <-rbind(cells, read.table(r[16],header=FALSE,sep='\t', col.names = c('ID', 'Value')))
    }
  }
}

best_per_group <- matrix(NA, length(detectors), ncol = 2)
i <- 1
for(d in detectors){
  best_per_group[i,1] <- d
  best_per_group[i,2] <- best[i]
  i <- i + 1
}

bg <- list(SI = SI,
           cells = cells,
           best_per_group = best_per_group)


#```
print(bg$best_per_group)
print(plot_voltrations(bg))
#```
dev.off()